#
# Anbox Cloud
# Copyright 2020 Canonical Ltd.  All rights reserved.
#
import json
import os
import ssl
from typing import Mapping
from http import client as httpclient

from charmhelpers.core.hookenv import log
from charms.reactive import Endpoint, when, when_not, set_flag, clear_flag


class LxdRequires(Endpoint):
    # {endpoint_name} will be filled in by the reactive framework. This is the
    # name of the endpoint as defined in `metadata.yaml`
    @when('endpoint.{endpoint_name}.changed')
    def changed(self):
        set_flag(self.expand_name('endpoint.{endpoint_name}.available'))
        clear_flag(self.expand_name('endpoint.{endpoint_name}.changed'))

    @when_not('endpoint.{endpoint_name}.joined')
    def broken(self):
        clear_flag(self.expand_name('endpoint.{endpoint_name}.available'))


    def _new_connection(self, endpoint: str, key_path: str, cert_path: str) -> httpclient.HTTPSConnection:
        """
        Return a http.client.HTTPSConnection configured with the proper endpoint and certificates
        """
        sslcontext = ssl.create_default_context(purpose=ssl.Purpose.CLIENT_AUTH)
        sslcontext.load_cert_chain(certfile=cert_path, keyfile=key_path)
        # Depending on how it was initialized, the LXD server cert can be configured
        # with 127.0.0.1 as its CN, failing the verification
        sslcontext.check_hostname = False
        return httpclient.HTTPSConnection(endpoint, context=sslcontext, timeout=5)


    def endpoints(self):
        nodes = []
        for relation in self.relations:
            for unit in relation.units:
                if not unit.received['nodes']:
                    continue

                for node in unit.received['nodes']:
                    nodes.append({
                        'endpoint': node['endpoint'],
                        'name': node['name'],
                        'cert_fingerprints': node['trusted_certs_fp']
                    })
        return nodes

    def register_cert(self, cert):
        """
        Publish client certificate to register it with LXD
        """
        clients_certs = self.relations[0].to_publish['client_certificates'] or []
        clients_certs.append(cert)
        self.relations[0].to_publish['client_certificates'] = list(set(clients_certs))
