#
# Anbox - The Android in a Box runtime environment
# Copyright 2018 Canonical Ltd.  All rights reserved.
#

import requests_unixsocket
import requests
import os
import time

import urllib.parse

AMS_UNIX_SOCKET = '/var/snap/ams/common/server/unix.socket'
SOCKET = 'http+unix://{}'.format(urllib.parse.quote_plus(AMS_UNIX_SOCKET))
SOCKET_NOT_AVAILABLE = 'socket not available'
SOCKET_WAIT_STEP = 5
SOCKET_WAIT_MAX = 3
session = requests_unixsocket.Session()


def change_group_ownership(grp):
    os.system('chgrp {} {}'.format(grp, AMS_UNIX_SOCKET))


def wait_for_socket_available():
    n = SOCKET_WAIT_MAX
    while n > 0:
        if os.path.exists(AMS_UNIX_SOCKET):
            break
        n = n - 1
        time.sleep(SOCKET_WAIT_STEP)

    if n <= 0:
        raise Exception(SOCKET_NOT_AVAILABLE)


def get(path):
    wait_for_socket_available()

    # Monkeypatch makes code easier, not having to create a Session
    # object and deal with it directly.
    # See:
    # https://github.com/msabramo/requests-unixsocket#implicit-monkeypatching
    with requests_unixsocket.monkeypatch():
        r = requests.get('{}/{}'.format(SOCKET, path))
        return r.status_code, r.text


def post(path, data):
    wait_for_socket_available()

    # Monkeypatch makes code easier, not having to create a Session
    # object and deal with it directly.
    # See:
    # https://github.com/msabramo/requests-unixsocket#implicit-monkeypatching
    with requests_unixsocket.monkeypatch():
        headers = {'Content-Type': 'application/json'}
        r = requests.post('{}/{}'.format(SOCKET, path),
                          data=data, headers=headers)
        return r.status_code, r.text


def delete(path):
    wait_for_socket_available()

    # Monkeypatch makes code easier, not having to create a Session
    # object and deal with it directly.
    # See:
    # https://github.com/msabramo/requests-unixsocket#implicit-monkeypatching
    with requests_unixsocket.monkeypatch():
        r = requests.delete('{}/{}'.format(SOCKET, path))
        return r.status_code, r.text
